#include <iostream>
#include <iomanip>
#include <fstream>
#include <math.h>
#include <stdio.h>
#include <cstdlib>
#include "cell.h"
using namespace std;

int main()
{
	//********************** Information on Hydroshed original data. Should not change ******************
	/* HydroSHED data for Asia, Flow Direction and DEM
	LL corner Lat =-10
	LL corner Lon =55
	UR corner Lat =60
	UR corner Lon =180	
	*/
	const int HyRow=8400;    // =((UR_LAT-LL_LAT)*120
	const int HyCol=15000;   // =((UR_LON-LL_LON)*120
	double  Dx = 0.00833333; // cellsize in degrees
	double  ULLat = 60-Dx/2; // The latitude of the center of Upper Left (North West) grid. 
	                         // 60 (degree) is the latitude of North West corner of Asia data of HydroSHED.
	double  ULLon = 55+Dx/2; // The longitude of the center of Upper Left (North West) grid. 
	                         // 55 (degree) is the longitude of North West corner of Asia data of HydroSHED.
	const char *projectName = "";   // Used if you creat project folder.
	const int forOneCell = 1;		// set always one
	//***************************************************************************************************

	char filename[256];
	const float NODATA = -9999;
	const int NODATA4BIN=-9999;

	//read study area for cliping topographic data.
	cout << "1) Reading location of study area." << endl;
	ifstream finClipArea("./clipArea.dat");
	if(!finClipArea){
		cout << "clipArea.dat cannot open." << endl;
		cout << endl;
		cout << "Program ends. Hit any key!" << endl;
		char ch;
		cin.get(ch);
		return 1;
	}

	// define the calculation region //  one grid size is =1/120 degrees.
	///*Chaopria thailand */
	//const int Row = 120*8;		// No of rows = degrees in latitude direction * 120
	//const int Col = 120*6;		// No of columns = degrees in Longitude direction * 120 
	//const int NWLAT = 21;			// Upper left corner Latitude (deg.)
	//const int NWLON = 97;			// Left corner longitude (deg.)

	int wRow, wCol, wNWLAT, wNWLON;

	finClipArea >> wRow; cout << "  Number of row : " << wRow << endl;
	finClipArea >> wCol; cout << "  Number of column : " << wCol << endl;
	finClipArea >> wNWLAT; cout << "  Latitude (degree) of North West corner : " << wNWLAT << endl;
	// Upper left corner Latitude (deg.)
	finClipArea >> wNWLON; cout << "  Longitude (degree) of North West corner : " << wNWLON << endl;
	// Left corner longitude (deg.)

	const int Row = wRow;
	const int Col = wCol;
	const int NWLAT = wNWLAT;
	const int NWLON = wNWLON;

	finClipArea.close();


	/*--------------------------------File Names-------------------------------------------------*/	
	FILE *HyDem;
	FILE *HyDir;
	errno_t error;

	//make output project directory
	//sprintf(filename,"mkdir ./output/%s",projectName);
	//system (filename);
	
	//memory allocation
	int** HyshedClipDem;
	HyshedClipDem = new int *[Row];
	for(int i=0; i < Row; i++){
		HyshedClipDem[i] = new int [Col];
	}
	int** HyshedClipDir;
	HyshedClipDir = new int *[Row];
	for(int i=0; i < Row; i++){
		HyshedClipDir[i] = new int [Col];
	}

	//read topographic data generated from HydroSHED
	if(error = fopen_s(&HyDem, "./input/as_dem_30s_clipped_55--10-180-60.bin", "rb") != NULL) {
		cout << "Hydroshed DEM cannot open." << endl;  //hunu
		exit(1);
	} 

	if(error = fopen_s(&HyDir, "./input/as_dir_30s_clipped_55--10-180-60.bin", "rb") != NULL) {
		cout << "Hydroshed Flow direction file cannot open." << endl;  //hunu
		exit(1);
	}

	// set clip area	
	short int idem, idir;
	int cut_i, cut_j;   // Upper left corner i and  j of the clipped box
	double tempLat, tempLon;
	tempLat=NWLAT-Dx/2;
	tempLon=NWLON+Dx/2;
	
	cut_i= (int) ((ULLat-tempLat)/Dx +0.5);
	cut_j= (int) ((tempLon-ULLon)/Dx +0.5);

	// read the Hydroshed data and assigned to an array
	cout << "2) Reading topography data." << endl;
	for (int i=0; i<HyRow; i++){
		if (i%100 == 0) cout << "  reading " << i << "/" << HyRow<< "th row data.\r";
		for (int j=0; j<HyCol; j++){
			fread(&idem, sizeof(short int), 1, HyDem);	
			fread(&idir, sizeof(short int), 1, HyDir);
			
			if (i>=cut_i && i<cut_i+Row && j>=cut_j && j<cut_j+Col) {
				HyshedClipDem[i-cut_i][j-cut_j]=idem;
				HyshedClipDir[i-cut_i][j-cut_j]=idir;
			}
		}
	}	
	fclose(HyDem);
	fclose(HyDir);
	
	//write the clipped data into ESRI ASCII file format
	sprintf_s(filename, 256, "./output/arcgis/cutDem.asc", projectName);
	ofstream foutDem(filename);
	if(!foutDem){
		cout << "cutdem.asc file cannot open \n";
		return 1;
	}

	//ESRI header
	foutDem << "ncols "<<Col<<"\n";
	foutDem << "nrows "<<Row<<"\n";
	foutDem << "xllcorner "<<NWLON<<"\n";
	foutDem << "yllcorner "<<NWLAT-(Row/120.0)<<"\n";
	foutDem << "cellsize "<<Dx<<"\n";
	foutDem << "NODATA_value "<< NODATA << "\n";

	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++){
			foutDem << " " << HyshedClipDem[i][j];
		}
		foutDem <<"\n";
	}
	cout << "  Writing " << filename << " done." <<endl;
	foutDem.close();

	sprintf_s(filename, 256, "./output/arcgis/cutDir.asc", projectName);	
	ofstream foutDir(filename);
	if(!foutDir){
		cout << "cutDir.asc file cannot open \n";
		return 1;
	}
	//ESRI header
	foutDir << "ncols "<<Col<<"\n";
	foutDir << "nrows "<<Row<<"\n";
	foutDir << "xllcorner "<<NWLON<<"\n";
	foutDir << "yllcorner "<<NWLAT-(Row/120.0)<<"\n";
	foutDir << "cellsize "<<Dx<<"\n";
	foutDir << "NODATA_value "<< NODATA << "\n";

	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++){
			foutDir << " " << HyshedClipDir[i][j];
		}
		foutDir <<"\n";
	}

	cout << "  Writing " << filename << " done." <<endl;
	foutDir.close();
	
    // data processing.
	cout << "3) Processing topography data." << endl;
	FILE *fCorrectedMed;
	FILE *fDown;
	FILE *fBasin;
	FILE *fRiverNumber;
	
	int enterKey;

	/*--------------------------------Memory allocation---------------------------------*/
	class Direction** cellObject;

	cout << "  Memory allocation." << endl;
	cellObject = new class Direction*[Row/forOneCell];
	for(int i=0; i<Row/forOneCell; i++){
		cellObject[i] = new class Direction[Col/forOneCell];
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			idem = HyshedClipDem[i][j];
			idir = HyshedClipDir[i][j];
			cellObject[i][j].setElevation(i, j, NWLAT, NWLON, forOneCell, 
				idem, Row/forOneCell, Col/forOneCell, idir); //hunu
		}
	}

	//hunu HydroSHED flow direction sign convention is diffrent from the convention used here 
	// convert Hydroshed direction
	cout << "  Assigning elevation data." << endl;

	for(int i=0; i<Row/forOneCell; i++){ 
		for(int j=0; j<Col/forOneCell; j++){

			if (cellObject[i][j].dir ==1){
				cellObject[i][j].dir=3; // EE: 3
			}
			else if (cellObject[i][j].dir ==2){
				cellObject[i][j].dir=4; // SE: 4
			}		
			else if (cellObject[i][j].dir ==4){
				cellObject[i][j].dir=5; // SS: 5
			}		
			else if (cellObject[i][j].dir ==8){
				cellObject[i][j].dir=6; // SW: 6
			}	
			else if (cellObject[i][j].dir ==16){
				cellObject[i][j].dir=7; // WW: 7
			}	
			else if (cellObject[i][j].dir ==32){
				cellObject[i][j].dir=8; // NW: 8
			}	
			else if (cellObject[i][j].dir ==64){
				cellObject[i][j].dir=1; // NN: 1
			}	
			else if (cellObject[i][j].dir ==128){
				cellObject[i][j].dir=2; // NE: 2
			}
			else if (cellObject[i][j].dir ==0){
				cellObject[i][j].dir=0; // river mouth: 0
				// ͂ȂB͌łB
			}				
			else if (cellObject[i][j].dir ==-1){
				cellObject[i][j].dir = -1; // sea: -1
				// According to HydroShed, this is a inland sink or pit. 
				// For this programme, this is set as a cell in the sea.
				// look at the function mouthAndHollow()   				
			}	
			else if (cellObject[i][j].dir ==-9999){ // nodata
				cellObject[i][j].dir=-1;
				// if nodata, the cell is treated as the sink.
			}				
			else {
				cout << i << "-" << j << "-" << cellObject[i][j].dir << "cellObject.down is wrong.\n";
				return 1;
			}

			if (cellObject[i][j].dem  < 0.0){ // Wł΂̒n_ sink Ƃ
				cellObject[i][j].dir = -1;
			}
		}
	}

	// This part (below) is to make the data at boundary as the sea even not surrounded by the sea	
	for(int i=0; i<Row/forOneCell; i++){ 
		for(int j=0; j<Col/forOneCell; j++){
			if (i==0 || j==0 || i== (Row-1) || j==(Col-1)){ // ؂[̃ObhEƍl
				cellObject[i][j].dir = -9; // set boundary as -9.
				cellObject[i][j].dem = -9999; // set elevation as sink.
			}
			if (i==1 || j==1 || i== (Row-2) || j==(Col-2)){ // ͌Ƃ
				cellObject[i][j].dir=0;
			}
		}
	}
	//This part (above) is to make data even the area is not surrounded by the sea

	// set the neighbor grid number
	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			cellObject[i][j].setNULL(); // initial setting
			cellObject[i][j].setNeighbor(cellObject, i, j);
		}	
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(cellObject[i][j].dir == -9) { // Eł͂̊O̔zɃANZXƃG[
				                              // ɂȂ̂ŃXLbvB
				continue;
			}
			if (cellObject[i][j].dir == -1){ // sink
				cellObject[i][j].SinkSet();
				//  sink ܂͊Cł́AObh̕W -9999 ƂAאڂ
				// 8̃Obh͌Ƃ dir=0 ƂB
			}
		}	
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			if(cellObject[i][j].dir < 0) {
				continue; 
			}
			if (i==0 || i==Row/forOneCell-1 || j==0 || j==Col/forOneCell-1) {
				continue; // skip at boundary
			}

			cellObject[i][j].inlandOrCoastal();	
		}
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			if(cellObject[i][j].dir < 0) {
				continue; 
			}
			if (i==0 || i==Row/forOneCell-1 || j==0 || j==Col/forOneCell-1) {
				continue; // skip at boundary
			}

			cellObject[i][j].compareSlopeOfMediumElevation(1);
			cellObject[i][j].mouthAndHollow();
		}
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			if (i==0 || i==Row/forOneCell-1 || j==0 || j==Col/forOneCell-1) {
				continue; // skip at boundary
			}

			if (cellObject[i][j].dir <= 0) {
				continue; 
			}
			else {
				cellObject[i][j].setDownStreamGrid();
			}
		}
	}

	/*--------------------------------Set Upper grid-----------------------------------------------*/
	cout << "  Setting upper grid number.\n";

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			//excluding the cells which dose not need to set upper cell
			// inland pit and sea
			if(cellObject[i][j].dir < 0) {
				cellObject[i][j].sizeOfBasin = -9999; //originally this was 0;
				continue;
			}
			// boundary
			if (i==0 || i==Row/forOneCell-1 || j==0 || j==Col/forOneCell-1) {
				continue;
			}
			// 
			if(cellObject[i][j].downStreamGrid == NULL) {
				continue;
			}
			cellObject[i][j].setUpStreamGrid();
			//cout << i<<"-"<<j<<"this is 5 \n";
		}
	}

	// hunu In HydroSHED flow direction map, all grids at the sea is set to "0".
    // but in this programme "0" should set only for the outlet of a basin.

	cout << "  Setting the outlet.\n";
	for (int i=0; i<Row/forOneCell; i++){
		for (int j=0; j<Col/forOneCell; j++){
			if (cellObject[i][j].dir == 0) {
				cellObject[i][j].setDownAtMouth(); 
			}
		}
	}

	
	/*--------------------------------Calculate the distance from the mouth---------------------------------------------*/
	cout << "  Calculating the distance from the outlet.\n";
	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(cellObject[i][j].dir==0){
				cellObject[i][j].distanceFromMouth=0;
			}
		}
	}

	int distanceNum=0;
	int numberOfNotSettingTime=0;

	while(numberOfNotSettingTime < Row/forOneCell*Col/forOneCell){

		numberOfNotSettingTime=0;

		for(int i=0; i<Row/forOneCell; i++){
			for(int j=0; j<Col/forOneCell; j++){
				if(cellObject[i][j].distanceFromMouth==distanceNum){
					cellObject[i][j].setDistanceFromMouth();
				}
				else{
					numberOfNotSettingTime++;
				}
			}
		}
		distanceNum++;
	}
	
	/*--------------------------------set basin size---------------------------------------------------*/
	cout << "  Calculating basin size.\n";
	for(int k=distanceNum; k>-1; k--){
		for(int i=0; i<Row/forOneCell; i++){
			for(int j=0; j<Col/forOneCell; j++){
				if(cellObject[i][j].distanceFromMouth==k){
					cellObject[i][j].setSizeOfBasinEx(); 
				}
			}
		}
	}

	/*--------------------------------riverNumber---------------------------------------------------*/
	cout << "  Setting river number.\n";

	int rNum;
	rNum = 1;

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(cellObject[i][j].dir == 0){
				cellObject[i][j].riverNumber = rNum;
				rNum++;
			}
		}
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			
			if(cellObject[i][j].sizeOfBasin == 1){ // ͉Ă̂
				//cout << i <<"-"<< j<<" this is 7 \n";
				cellObject[i][j].numberRiverNumber();
			}
		}
	}


	/*--------------------------------writing output files---------------------------------------------*/
	cout <<"4) Writing output files" << endl;

	sprintf_s(filename, 256, "./output/modDem.bin", projectName);
	if(error = fopen_s(&fCorrectedMed, filename, "wb") != NULL) {
		cout << "modDem.bin file cannot open." << endl;
		exit(1);
	} 

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			fwrite(&cellObject[i][j].dem, sizeof(double), 1, fCorrectedMed);  // write 8 bite real value
		}
	}
	cout << "  Writing " << filename <<endl;
	fclose(fCorrectedMed);
	
	/*----------------------------modified DEM --------------------------------------------*/
	enterKey = NWLON*3600 + forOneCell*30*1/2;
	
	double xllcorner, yllcorner;
	xllcorner=NWLON;
	yllcorner=NWLAT-(Row/120.0);
		
	sprintf_s(filename, 256, "./output/arcgis/modDem.asc", projectName);
	ofstream fout3(filename);
	if(!fout3){
		cout << "modDem.asc file can not open\n";
		return 1;
	}
	//ESRI header

	fout3 << "ncols "<<Col<<"\n";
	fout3 << "nrows "<<Row<<"\n";
	fout3 << "xllcorner "<<xllcorner<<"\n";
	fout3 << "yllcorner "<<yllcorner<<"\n";
	fout3 << "cellsize "<<Dx<<"\n";
	fout3 << "NODATA_value "<< NODATA;

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(j == 0) {
				fout3 << "\n";
			}
			fout3 << " " << cellObject[i][j].dem;	
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fout3.close();


	/*--------------------------------Distance from the sea --------------------------------------------*/
	/*  This information is not used in the Runoff mode */

	sprintf_s(filename, 256, "./output/arcgis/distance.asc", projectName);
	ofstream fout2(filename);
	if(!fout2){
		cout << "dstance.asc file can not open\n";
		return 1;
	}
	//ESRI header
	fout2 << "ncols "<<Col<<"\n";
	fout2 << "nrows "<<Row<<"\n";
	fout2 << "xllcorner "<<xllcorner<<"\n";
	fout2 << "yllcorner "<<yllcorner<<"\n";
	fout2 << "cellsize "<<Dx<<"\n";
	fout2 << "NODATA_value "<<-1;

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(j == 0) {
				fout2 << "\n";
			}
			fout2 << " " << cellObject[i][j].distanceFromMouth;
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fout2.close();


	//--------------------------------Flow direction file-------------------------------------------

	sprintf_s(filename, 256, "./output/flowDir.bin", projectName);
	if(error = fopen_s(&fDown, filename, "wb") != NULL) {
		cout << "flowDir.bin file cannot open." << endl;
		exit(1);
	} 
	
	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			if (i==0 || i==Row/forOneCell-1 || j==0 || j==Col/forOneCell-1) {
				fwrite(&NODATA4BIN, sizeof(int), 1, fDown); // write 4 bite integer value
				continue; // skip at boundary
			}
			fwrite(&cellObject[i][j].dir, sizeof(int), 1, fDown); // write 4 bite integer value
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fclose(fDown);

	// ArcGis 
	sprintf_s(filename, 256, "./output/arcgis/flowDir.asc", projectName);
	ofstream fout(filename);
	if(!fout){
		cout << "file can not open\n";
		return 1;
	}
	//ESRI header
	fout << "ncols "<<Col<<"\n";
	fout << "nrows "<<Row<<"\n";
	fout << "xllcorner "<<xllcorner<<"\n";
	fout << "yllcorner "<<yllcorner<<"\n";
	fout << "cellsize "<<Dx<<"\n";
	fout << "NODATA_value "<< NODATA;

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(j == 0) {
				fout << "\n";
			}

			// change the Flow direction to ESRI arc grid format
			if (cellObject[i][j].dir ==3){
				cellObject[i][j].dir = 1;
			}
			else if (cellObject[i][j].dir ==4){
				cellObject[i][j].dir=2;
			}		
			else if (cellObject[i][j].dir ==5){
				cellObject[i][j].dir=4;
			}		
			else if (cellObject[i][j].dir ==6){
				cellObject[i][j].dir=8;
			}	
			else if (cellObject[i][j].dir ==7){
				cellObject[i][j].dir=16;
			}	
			else if (cellObject[i][j].dir ==8){
				cellObject[i][j].dir=32;
			}	
			else if (cellObject[i][j].dir ==1){
				cellObject[i][j].dir=64;
			}	
			else if (cellObject[i][j].dir ==2){
				cellObject[i][j].dir=128;
			}
			else if (cellObject[i][j].dir ==0){
				cellObject[i][j].dir=0;
			}				
			else if (cellObject[i][j].dir < 0) { 
				cellObject[i][j].dir=-9999;  
			}				
			else {
				cout << i <<"-"<< j << "-"<< cellObject[i][j].dir << "cellObject.down is wrong\n";
			}
			fout << " " << cellObject[i][j].dir;
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fout.close();
	

	/* Flow accumulation file to visualize on GRADS 
	FILE *ffacc;
	float FaccGrad;

	sprintf_s(filename, 256, "./output/flowAccGrads.bin", projectName);
	if(error = fopen_s(&ffacc, filename, "wb") != NULL) {
		cout << "faccgrads.bin file cannot open." << endl;
		exit(1);
	} 

	for(int i=Row/forOneCell-1; i>=0; i--){
		for(int j=0; j<Col/forOneCell; j++){
			
			if (i==0 || i==Row/forOneCell-1 || j==0 || j==Col/forOneCell-1) {
				fwrite(&NODATA, sizeof(float), 1, ffacc); // write 4 bite integer value
				continue; // skip at boundary
			}
            FaccGrad = (float)cellObject[i][j].sizeOfBasin;
			fwrite(&FaccGrad, sizeof(float), 1, ffacc); // write 4 bite real value
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fclose(ffacc);
	// Above file is the Flow accumulation file to visualize  on GRADS */


	//-------------------------Flow accumulation file--------------------------------------------
	sprintf_s(filename, 256, "./output/flowAcc.bin", projectName);
	if(error = fopen_s(&fBasin, filename, "wb") != NULL) {
		cout << "flowAcc.bin file cannot open." << endl;
		exit(1);
	} 

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			
			if (i==0 || i==Row/forOneCell-1 || j==0 || j==Col/forOneCell-1) {
				fwrite(&NODATA4BIN, sizeof(int), 1, fBasin); // write 4 bite integer value
				continue; // skip at boundary
			}
			fwrite(&cellObject[i][j].sizeOfBasin, sizeof(int), 1, fBasin); // write 4 bite integer value
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fclose(fBasin);

	// ArcGIS 
	sprintf_s(filename, 256, "./output/arcgis/flowAcc.asc", projectName);
	ofstream fout1(filename);
	if(!fout1){
		cout << "flowAcc.asc file can not open\n";
		return 1;
	}
	fout1 << "ncols "<<Col<<"\n";
	fout1 << "nrows "<<Row<<"\n";
	fout1 << "xllcorner "<<xllcorner<<"\n";
	fout1 << "yllcorner "<<yllcorner<<"\n";
	fout1 << "cellsize "<<Dx<<"\n";
	fout1 << "NODATA_value "<< NODATA;

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(j == 0) {
				fout1 << "\n";
			}
			if (i==0 || i==Row/forOneCell-1 || j==0 || j==Col/forOneCell-1) {
				fout1 << " " << NODATA;
				continue;
			}
			fout1 << " " << cellObject[i][j].sizeOfBasin;
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fout1.close();

	/*--------------------------------riverNumber---------------------------------------------*/
	sprintf_s(filename, 256, "./output/riverNum.bin", projectName);
	if(error = fopen_s(&fRiverNumber, filename, "wb") != NULL) {
		cout << "riverNum.bin file cannot open." << endl;
		exit(1);
	} 

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if (i==0 || i==Row/forOneCell-1 || j==0 || j==Col/forOneCell-1) {
				fwrite(&NODATA4BIN, sizeof(int), 1, fRiverNumber); // write 4 bite integer value
				continue; // skip at boundary
			}
			fwrite(&cellObject[i][j].riverNumber, sizeof(int), 1, fRiverNumber); // write 4 bite integer value
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fclose(fRiverNumber);

	// ArcGIS
	sprintf_s(filename, 256, "./output/arcgis/riverNum.asc", projectName);
	ofstream fout4(filename);
	if(!fout4){
		cout << "ESRIRiverNum.asc riverNumber file can not open\n";
		return 1;
	}
	fout4 << "ncols "<<Col<<"\n";
	fout4 << "nrows "<<Row<<"\n";
	fout4 << "xllcorner "<<xllcorner<<"\n";
	fout4 << "yllcorner "<<yllcorner<<"\n";
	fout4 << "cellsize "<<Dx<<"\n";
	fout4 << "NODATA_value "<<-1 ;
	
	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(j == 0) {
				fout4 << "\n";
			}
			if (i==0 || i==Row/forOneCell-1 || j==0 || j==Col/forOneCell-1) {
				fout4 << " " << -1;
				continue;
			}
			fout4 << " " << cellObject[i][j].riverNumber;
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fout4.close();

	//--- riverNumber list ŉƂȂObh̏o
	sprintf_s(filename, 256, "./output/riverNumList.dat", projectName);
	ofstream fRiverNumList(filename);
	if(!fRiverNumList){
		cout << "riverNumList.dat file can not open\n";
		return 1;
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(cellObject[i][j].dir==0){
				fRiverNumList << cellObject[i][j].riverNumber 
					<< " " << i+1 
					<< " " << j+1 
					<< " " << cellObject[i][j].longitude
					<< " " << cellObject[i][j].latitude
					<< " " << cellObject[i][j].sizeOfBasin
					<< endl;
			}
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fRiverNumList.close();

	delete[] cellObject;

	cout << endl;
	cout << "Program ends. Hit any key!" << endl;
	char ch;
	cin.get(ch);
	
	return 0;
}



void Direction::compareSlopeOfMediumElevation(int number)
{
	lenOf30SecLat = 926.6;

	SteepestSlope = -50000;
	numberOfDirection = 0;
	
	for(int i=0; i<8; i++){
		if(neighborGrid[i]->dem == -9999)
		continue;

		longiudinalDistance = longitudinalLength * abs(longitude - neighborGrid[i]->longitude)/30.0;
		latitudinalDistance = lenOf30SecLat * abs(latitude - neighborGrid[i]->latitude)/30.0;

		distance = sqrt(pow(latitudinalDistance, 2.0) + pow(longiudinalDistance, 2.0));

		Slope[i] = (dem - neighborGrid[i]->dem)/distance;

		if(SteepestSlope < Slope[i]){
			SteepestSlope = Slope[i];
			// hunu dir = i + 1;
		}
	}

	for(int i=0; i<8; i++){
		if(SteepestSlope == Slope[i]){
			Number[i] = number;	
			numberOfDirection++; 
			// if this numberOfDirection >1 , flow from the grid can goes to 
			// several (numberOfDirection) grids
		}			
	}
}



void Cell::setElevation(int rowStep, int colStep, int lat, int lon, int numberOfGrid,  int medElevation, int nRow, int nCol, int dir1)//hunu
{
	latitude  = lat*3600 - 30*numberOfGrid*rowStep - 15*numberOfGrid;
	longitude = lon*3600 + 30*numberOfGrid*colStep + 15*numberOfGrid;

	longitudinalLength = 926.6*cos(latitude/3600.0/180.0*3.14);
	//if (rowStep >0 && colStep > 0 && rowStep <nRow-1 && colStep< nCol-1){

	dem = medElevation;
	
	dir = dir1; //hunu
	//}
	//else {
	
	//	dem = -9999;
	
	//}
}

/*  Initialize flow direction, adjesant grid number and lower grid number */
void Cell::setNULL()
{
	//hunu down=10; 
	distanceFromMouth=-1;
	sizeOfBasin=1;
	riverNumber=-1;

	downStreamGrid=NULL;
	startOFsetSizeOfBasin=false;

	for(int i=0; i<8; i++){
		neighborGrid[i]=NULL;
	}

	for(int i=0; i<7; i++){
		upStreamGrid[i]=NULL;
	}
}


void Cell::setNeighbor(class Direction** cell2, int i, int j)
{	
	neighborGrid[0]=&cell2[i-1][j];
	neighborGrid[1]=&cell2[i-1][j+1];
	neighborGrid[2]=&cell2[i][j+1];
	neighborGrid[3]=&cell2[i+1][j+1];
	neighborGrid[4]=&cell2[i+1][j];
	neighborGrid[5]=&cell2[i+1][j-1];
	neighborGrid[6]=&cell2[i][j-1];
	neighborGrid[7]=&cell2[i-1][j-1];
}

void Cell::SinkSet()
{
	dem = -9999; // set the elevation of the sink as sea
	for(int i=0; i<8; i++){
		neighborGrid[i]->dir=0; // set the surrounding grids as river mouth.
	}

}

// hunu In HydroSHED flow direction map, all grids at the sea is set to "0".
// but in this programme "0" should set only for the outlet of a basin.
void Cell::setDownAtMouth()
{
	int hhh=0;
	for(int i=0; i<7; i++){
		if(upStreamGrid[i]!=NULL){
			hhh=1;
		}
	}
	if (hhh==0){
		dir = -1;
		dem = -9999;
		sizeOfBasin = -1;
		//cout<<"this is 8\n";
	} 
}

void Direction::inlandOrCoastal() // ΏۃObh̎̊Č𐔂B
{
	numberOfCoastalGrid=0;

	for(int i=0; i<8; i++){

		Number[i] = 0;
		
		if(neighborGrid[i]->dem == -9999)
		numberOfCoastalGrid++;
	}
}

//delete this function
void Direction::reclamationOfHollow(int* g9)
{
	lowElevation = 9999;

	double medDummy;
	medDummy = dem;


	for(int i=0; i<8; i++){
		if(lowElevation > neighborGrid[i]->dem)
		lowElevation = neighborGrid[i]->dem;
	}

	if(dem <= lowElevation){
		dem = lowElevation+0.1;
		*g9 = *g9+1;
	}
}

void Direction::mouthAndHollow()
{
	if(SteepestSlope <= 0){
		if(numberOfCoastalGrid == 0){		
			
			for(int i=0; i>8; i++){
				cout << neighborGrid[i]->dem << "check the elevation \n";
			}
		}
		else{
			//dir = 0; // this cell flows to the sea
		}					
	}
}


void Direction::setDownStreamGrid()
{

	if(1<=dir && dir <=8){
		downStreamGrid = neighborGrid[dir-1];
	}
	else{
		downStreamGrid = NULL;
	}
}

/* */
void Direction::setUpStreamGrid()
{
	// int thisdown =99;
	for(int i=0; i<7; i++){
		if(downStreamGrid->upStreamGrid[i]==NULL){ 
			// cellobject(i,j) wala downstream grid ekee upstream grid eka  cellobject(i,j) wiyautui
			downStreamGrid->upStreamGrid[i]=this;
			//if (downStreamGrid->dem==-9999){
			//downStreamGrid->down=0;
			//thisdown=downStreamGrid->down;
			//cout <<thisdown <<" this is \n";
			//}
			break;
		}
	}
}

/* calculate the distance from the river mouth */
void Direction::setDistanceFromMouth()
{
	for(int i=0; i<7; i++){
		if(upStreamGrid[i]!=NULL){
			upStreamGrid[i]->distanceFromMouth = distanceFromMouth + 1;
		}
	}				
}

void Direction::setSizeOfBasinEx()
{	
	for(int i=0; i<7; i++){

		if(upStreamGrid[i]!=NULL){
			sizeOfBasin += upStreamGrid[i]->sizeOfBasin;// initially sizeOfBasin is set to 1 see SetNULL function
		}
	}
}

/* riverNumber */
void Direction::numberRiverNumber()
{
	int rNum;
	class Cell* object;
	
	
	object = this;

	while(object->riverNumber==-1){
		
		if(object->downStreamGrid==NULL)
		break;

		object = object->downStreamGrid;
	}
	rNum = object->riverNumber;

	object = this;

	while(object->riverNumber==-1){
		
		if(object->downStreamGrid==NULL)
		break;

		object->riverNumber = rNum;
		object = object->downStreamGrid;
	}

}


